@@ -55,7 +55,8 @@ sub init {
     }
 
     unless ($build->{MP_USE_DSO} or $build->{MP_USE_STATIC}) {
-        $build->{MP_USE_DSO} = $build->{MP_USE_STATIC} = 1;
+        # default to DSO
+        $build->{MP_USE_DSO} = 1;
     }
 
     $build->{MP_GENERATE_XS} = 1 unless exists $build->{MP_GENERATE_XS};
@@ -221,7 +222,6 @@ STATIC_EXTS    0    Build Apache::*.xs as static extensions
 APXS           0    Path to apxs
 AP_PREFIX      0    Apache installation or source tree prefix
 AP_CONFIGURE   0    Apache ./configure arguments
-AP_BUILD       0    Whether to build httpd
 APR_CONFIG     0    Path to apr-config
 XS_GLUE_DIR    1    Directories containing extension glue
 INCLUDE_DIR    1    Add directories to search for header files
