@@ -46,18 +46,17 @@ request_rec *modperl_global_request(pTHX_ SV *svr)
 {
     request_rec *cur = NULL;
     apr_status_t status = modperl_tls_get_request_rec(&cur);
-
     if (status != APR_SUCCESS) {
         /* an internal problem */
         Perl_croak(aTHX_ "failed to retrieve the request object");
     }
 
-    if (!cur) {
+    if (GIMME_V != G_VOID && !cur) {
         /* wrong configuration */
         Perl_croak(aTHX_ "Global $r object is not available. Set:\n"
                    "\tPerlOptions +GlobalRequest\nin httpd.conf");
     }
-    
+
     if (svr) {
         modperl_global_request_obj_set(aTHX_ svr);
     }
@@ -165,7 +164,6 @@ void modperl_global_set_##gname(void *data)              \
 
 MP_GLOBAL_IMPL(pconf, apr_pool_t *);
 MP_GLOBAL_IMPL(server_rec, server_rec *);
-MP_GLOBAL_IMPL(threaded_mpm, IV);
 
 #if MP_THREADED
 static apr_status_t modperl_tls_cleanup(void *data)
