@@ -50,16 +50,18 @@ void *modperl_global_get(modperl_global_t *global);
 
 void modperl_global_set(modperl_global_t *global, void *data);
 
-#define MP_GLOBAL_DECL(gname, type) \
-void modperl_global_init_##gname(apr_pool_t *p, type gname); \
-void modperl_global_lock_##gname(void); \
-void modperl_global_unlock_##gname(void); \
-type modperl_global_get_##gname(void); \
-void modperl_global_set_##gname(void *)
+#define MP_GLOBAL_DECL(gname, type)                                \
+    void modperl_global_init_##gname(apr_pool_t *p, type gname);   \
+    void modperl_global_lock_##gname(void);                        \
+    void modperl_global_unlock_##gname(void);                      \
+    type modperl_global_get_##gname(void);                         \
+    void modperl_global_set_##gname(void *)
 
+/* modperl_global_get_pconf returns a thread un-safe object */
 MP_GLOBAL_DECL(pconf, apr_pool_t *);
+
+/* modperl_global_get_server_rec returns a thread un-safe object */
 MP_GLOBAL_DECL(server_rec, server_rec *);
-MP_GLOBAL_DECL(threaded_mpm, IV);
 
 apr_status_t modperl_tls_create(apr_pool_t *p, modperl_tls_t **key);
 apr_status_t modperl_tls_get(modperl_tls_t *key, void **data);
