@@ -119,7 +119,9 @@ modperl_interp_t *modperl_interp_new(modperl_interp_pool_t *mip,
 #endif
     }
 
-    MP_TRACE_i(MP_FUNC, "0x%lx\n", (unsigned long)interp);
+    MP_TRACE_i(MP_FUNC, "0x%lx / perl: 0x%lx / parent perl: 0x%lx\n",
+               (unsigned long)interp, (unsigned long)interp->perl,
+               (unsigned long)perl);
 
     return interp;
 }
@@ -131,8 +133,8 @@ void modperl_interp_destroy(modperl_interp_t *interp)
 
     PERL_SET_CONTEXT(interp->perl);
 
-    MP_TRACE_i(MP_FUNC, "interp == 0x%lx\n",
-               (unsigned long)interp);
+    MP_TRACE_i(MP_FUNC, "interp == 0x%lx / perl: 0x%lx\n",
+               (unsigned long)interp, (unsigned long)interp->perl);
 
     if (MpInterpIN_USE(interp)) {
         MP_TRACE_i(MP_FUNC, "*error - still in use!*\n");
@@ -246,7 +248,7 @@ void modperl_interp_init(server_rec *s, apr_pool_t *p,
 
     MP_TRACE_i(MP_FUNC, "server=%s\n", modperl_server_desc(s, p));
     
-    if (scfg->threaded_mpm) {
+    if (modperl_threaded_mpm()) {
         mip->tipool = modperl_tipool_new(p, scfg->interp_pool_cfg,
                                          &interp_pool_func, mip);
     }
@@ -339,7 +341,7 @@ modperl_interp_t *modperl_interp_pool_select(apr_pool_t *p,
     MP_dSCFG(s);
     modperl_interp_t *interp = NULL;
 
-    if (scfg && (is_startup || !scfg->threaded_mpm)) {
+    if (scfg && (is_startup || !modperl_threaded_mpm())) {
         MP_TRACE_i(MP_FUNC, "using parent interpreter at %s\n",
                    is_startup ? "startup" : "request time (non-threaded MPM)");
 
@@ -381,7 +383,7 @@ modperl_interp_t *modperl_interp_select(request_rec *r, conn_rec *c,
     int is_subrequest = (r && r->main) ? 1 : 0;
     modperl_interp_scope_e scope;
 
-    if (!scfg->threaded_mpm) {
+    if (!modperl_threaded_mpm()) {
         MP_TRACE_i(MP_FUNC,
                    "using parent 0x%lx for non-threaded mpm (%s:%d)\n",
                    (unsigned long)scfg->mip->parent,
