@@ -1,3 +1,18 @@
+/* Copyright 2001-2004 The Apache Software Foundation
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
+
 #include "mod_perl.h"
 
 /*
@@ -6,16 +21,16 @@
  * need config structures to be free of Perl structures
  */
 
-#define modperl_mgv_new_w_name(mgv, p, n, copy) \
-mgv = modperl_mgv_new(p); \
-mgv->len = strlen(n); \
-mgv->name = (copy ? apr_pstrndup(p, n, mgv->len) : n)
+#define modperl_mgv_new_w_name(mgv, p, n, copy)         \
+    mgv = modperl_mgv_new(p);                           \
+    mgv->len = strlen(n);                               \
+    mgv->name = (copy ? apr_pstrndup(p, n, mgv->len) : n)
 
-#define modperl_mgv_new_name(mgv, p, n) \
-modperl_mgv_new_w_name(mgv, p, n, 1)
+#define modperl_mgv_new_name(mgv, p, n)         \
+    modperl_mgv_new_w_name(mgv, p, n, 1)
 
-#define modperl_mgv_new_namen(mgv, p, n) \
-modperl_mgv_new_w_name(mgv, p, n, 0)
+#define modperl_mgv_new_namen(mgv, p, n)        \
+    modperl_mgv_new_w_name(mgv, p, n, 0)
 
 int modperl_mgv_equal(modperl_mgv_t *mgv1,
                       modperl_mgv_t *mgv2)
@@ -40,13 +55,13 @@ modperl_mgv_t *modperl_mgv_new(apr_pool_t *p)
     return (modperl_mgv_t *)apr_pcalloc(p, sizeof(modperl_mgv_t));
 }
 
-#define modperl_mgv_get_next(mgv) \
-    if (mgv->name) { \
-        mgv->next = modperl_mgv_new(p); \
-        mgv = mgv->next; \
+#define modperl_mgv_get_next(mgv)               \
+    if (mgv->name) {                            \
+        mgv->next = modperl_mgv_new(p);         \
+        mgv = mgv->next;                        \
     }
 
-#define modperl_mgv_hash(mgv) \
+#define modperl_mgv_hash(mgv)                   \
     PERL_HASH(mgv->hash, mgv->name, mgv->len)
  /* MP_TRACE_h(MP_FUNC, "%s...hash=%ld\n", mgv->name, mgv->hash) */
 
@@ -84,31 +99,6 @@ modperl_mgv_t *modperl_mgv_compile(pTHX_ apr_pool_t *p,
     return symbol;
 }
 
-char *modperl_mgv_name_from_sv(pTHX_ apr_pool_t *p, SV *sv)
-{
-    char *name = NULL;
-    GV *gv;
-
-    if (SvROK(sv)) {
-        sv = SvRV(sv);
-    }
-
-    switch (SvTYPE(sv)) {
-      case SVt_PV:
-        name = SvPVX(sv);
-        break;
-      case SVt_PVCV:
-        if (CvANON((CV*)sv)) {
-            Perl_croak(aTHX_ "anonymous handlers not (yet) supported");
-        }
-        gv = CvGV((CV*)sv);
-        name = apr_pstrcat(p, HvNAME(GvSTASH(gv)), "::", GvNAME(gv), NULL);
-        break;
-    };
-
-    return name;
-}
-
 void modperl_mgv_append(pTHX_ apr_pool_t *p, modperl_mgv_t *symbol,
                         const char *name)
 {
@@ -181,36 +171,10 @@ MP_INLINE GV *modperl_mgv_lookup_autoload(pTHX_ modperl_mgv_t *symbol,
 }
 #endif
 
-
-static void package2filename(apr_pool_t *p, const char *package,
-                             char **filename, int *len)
-{
-    const char *s;
-    char *d;
-
-    *filename = apr_palloc(p, (strlen(package)+4)*sizeof(char));
-
-    for (s = package, d = *filename; *s; s++, d++) {
-        if (*s == ':' && s[1] == ':') {
-            *d = '/';
-            s++;
-        }
-        else {
-            *d = *s;
-        }
-    }
-    *d++ = '.';
-    *d++ = 'p';
-    *d++ = 'm';
-    *d   = '\0';
-
-    *len = d - *filename;
-}
-
 /* currently used for complex filters attributes parsing */
 /* XXX: may want to generalize it for any handlers */
-#define MODPERL_MGV_DEEP_RESOLVE(handler, p) \
-    if (handler->attrs & MP_FILTER_HAS_INIT_HANDLER) { \
+#define MODPERL_MGV_DEEP_RESOLVE(handler, p)                   \
+    if (handler->attrs & MP_FILTER_HAS_INIT_HANDLER) {         \
         modperl_filter_resolve_init_handler(aTHX_ handler, p); \
     }
 
@@ -223,6 +187,11 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
     char *handler_name = "handler";
     char *tmp;
 
+    if (MpHandlerANON(handler)) {
+        /* already resolved anonymous handler */
+        return 1;
+    }
+    
     if (strnEQ(name, "sub ", 4)) {
         MP_TRACE_h(MP_FUNC, "handler is anonymous\n");
         MpHandlerANON_On(handler);
@@ -286,30 +255,28 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
     }
 
     if (!stash && MpHandlerAUTOLOAD(handler)) {
-        int len;
-        char *filename;
-        SV **svp;
-
-        package2filename(p, name, &filename, &len);
-        svp = hv_fetch(GvHVn(PL_incgv), filename, len, 0);
-
-        if (!(svp && *svp != &PL_sv_undef)) { /* not in %INC */
+        if (!modperl_perl_module_loaded(aTHX_ name)) { /* not in %INC */
             MP_TRACE_h(MP_FUNC,
-                       "package %s not in %INC, attempting to load '%s'\n",
-                       name, filename);
+                       "package %s not in %INC, attempting to load it\n",
+                       name);
 
-            if (modperl_require_module(aTHX_ name, FALSE)) {
+            if (modperl_require_module(aTHX_ name, logfailure)) {
                 MP_TRACE_h(MP_FUNC, "loaded %s package\n", name);
             }
             else {
-                MP_TRACE_h(MP_FUNC, "failed to load %s package\n", name);
-                return 0;
+                if (logfailure) {
+                    /* the caller doesn't handle the error checking */
+                    Perl_croak(aTHX_ "failed to load %s package\n", name);
+                }
+                else {
+                    /* the caller handles the error checking */
+                    MP_TRACE_h(MP_FUNC, "failed to load %s package\n", name);
+                    return 0;
+                }
             }
         }
         else {
-            MP_TRACE_h(MP_FUNC, "package %s seems to be loaded\n"
-                       "  $INC{%s)='%s';\n",
-                       name, filename, SvPV_nolen(*svp));
+            MP_TRACE_h(MP_FUNC, "package %s seems to be loaded\n", name);
         }
     }
 
@@ -318,8 +285,7 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
      * module was loaded, preventing from loading the module
      */
     if (!(stash || (stash = gv_stashpv(name, FALSE)))) {
-        MP_TRACE_h(MP_FUNC, "package %s seems to be loaded, "
-                   "but can't find its stash\n", name);
+        MP_TRACE_h(MP_FUNC, "%s's stash is not found\n", name);
         return 0;
     }
 
@@ -346,6 +312,14 @@ int modperl_mgv_resolve(pTHX_ modperl_handler_t *handler,
         return 1;
     }
 
+    /* at least modperl_hash_handlers needs to verify that an
+     * autoloaded-marked handler needs to be loaded, since it doesn't
+     * check success failure, and handlers marked to be autoloaded are
+     * the same as PerlModule and the failure should be fatal */
+    if (MpHandlerAUTOLOAD(handler)) {
+        Perl_croak(aTHX_ "failed to resolve handler %s\n", name);
+    }
+    
 #ifdef MP_TRACE
     /* complain only if the class was actually loaded/created */
     if (stash) {
@@ -417,7 +391,8 @@ int modperl_mgv_require_module(pTHX_ modperl_mgv_t *symbol,
 }
 #endif
 
-/* precompute the hash(es) for handler names */
+/* precompute the hash(es) for handler names, preload handlers
+ * configured to be autoloaded */
 static void modperl_hash_handlers(pTHX_ apr_pool_t *p, server_rec *s,
                                   MpAV *entry, void *data)
 {
