@@ -20,7 +20,7 @@ sub MyTest6 {
     my($self, $parms, $arg) = @_;
     $self->{+KEY} = $arg;
     unless ($parms->path) {
-        my $srv_cfg = Apache::Module->get_config($self, $parms->server);
+        my $srv_cfg = Apache::Module::get_config($self, $parms->server);
         $srv_cfg->{+KEY} = $arg;
     }
 }
@@ -38,7 +38,7 @@ use Apache::TestUtil;
 use Apache::Const -compile => qw(OK);
 
 sub get_config {
-    Apache::Module->get_config(__PACKAGE__, @_);
+    Apache::Module::get_config(__PACKAGE__, @_);
 }
 
 sub handler {
